from functions_authorization import get_function_authorization_level_from_config
from get_keyboard_comments_config import get_keyboard_comments_config
from get_min_value_to_ask_bordereau_justify_config import get_min_value_to_ask_bordereau_justify_config
from show_numpad_for_customer_name import show_numpad_for_customer_name
from get_production_config import get_hold_time_config
from should_ask_pager_number import should_ask_pager_number
from should_ask_delivery_type import should_ask_delivery_type
from get_default_delivery_type import get_default_delivery_type
from can_cancel_tef_payment_on_error import can_cancel_tef_payment_on_error



from logging import Logger

from model.error_type import PosValidationErrorsType
from interactor.route.order.cancel.dto import OrderCancelRequest
from model.config import Config
from model.exception import ValidationError
from model.repository import PosRepositoryModel
from validation_util import \
    check_if_business_day_is_opened, \
    check_if_operator_is_logged, \
    validate_api_key


class OrderCancelInteractor(object):
    def __init__(self, logger, config, pos_repository):
        # type: (Logger, Config, PosRepositoryModel) -> None

        self.logger = logger
        self.config = config
        self.pos_repository = pos_repository

    def execute(self, request):
        # type: (OrderCancelRequest) -> None

        pos_id = validate_api_key(request, self.config.api_key_mapping)

        model = self.pos_repository.get_model(pos_id)

        check_if_business_day_is_opened(self.pos_repository, model)
        check_if_operator_is_logged(self.pos_repository, model)

        if not self.pos_repository.has_current_order(model):
            raise ValidationError(PosValidationErrorsType.OrderNotFound)

        pos_ot = self.pos_repository.get_pos_ot(model)
        self.pos_repository.void_order(
            pos_id=pos_id,
            pos_ot=pos_ot,
            order_id=request.order_id,
        )



from ._EventData import EventData


class OrderPictureRetransmit(object):
    def __init__(
        self,          # type: OrderPictureRetransmit
        event_data,    # type: EventData
        payload,       # type: str
    ):
        # type: (...) -> None

        self.event_data = event_data
        self.payload = payload



from helper import ExtendedEnum

from messagebus import (
    TokenCreator,
    TokenPriority,
)

MSGGRP_PUMP = "B"


class ListenedTokens(ExtendedEnum):
    TK_PUMP_RETRANSMIT_ORDERS = TokenCreator.create_token(
        priority=TokenPriority.low,
        message_group=MSGGRP_PUMP,
        number="1",
    )



class MissingLogoutInfo(Exception):

    def __init__(
        self,               # type: MissingLogoutInfo
        business_period,    # type: str
    ):
        # type: (...) -> None
        super(MissingLogoutInfo, self).__init__("Missing logout info. Dispatched [MISSING_LOGOUT_INFO] event")

        self.business_period = business_period



from typing import Optional

from httpmessageprocessorutil.automapped import auto_map


@auto_map({"system_api_key": {"name": "X-Api-Key", "location": "header", "type": str},
           "pos_id": {"name": "posId", "type": str},
           "print_coupon": {"name": "printCoupon", "type": Optional[str]},
           "print_tef_customer_receipt": {"name": "printTefCustomerReceipt", "type": Optional[str]},
           "order_id": {"name": "orderId", "type": Optional[str]},
           "device_type": {"name": "deviceType", "type": Optional[str]}})
class OrderFinalizeRequest(object):
    def __init__(
        self,
        system_api_key,                         # type: str
        pos_id,                                 # type: str
        print_coupon='True',                    # type: Optional[str]
        print_tef_customer_receipt='False',     # type: Optional[str]
        order_id="",                            # type: Optional[str]
        device_type="",                         # type: Optional[str]
    ):
        # type: (...) -> None

        self.system_api_key = system_api_key
        self.pos_id = pos_id
        self.print_coupon = True if print_coupon is None else print_coupon.lower() == 'true'
        print_receipt = False if print_tef_customer_receipt is None else print_tef_customer_receipt.lower() == 'true'
        self.print_tef_customer_receipt = print_receipt
        self.order_id = order_id
        self.device_type = device_type.upper() if device_type is not None else ""




import json

from messagebus import (
    AckMessage,
    Event,
    Message,
    MessageBus,
    NakMessage,
)
from messageprocessorutil import UuidMessageProcessor
from typing import Any

from services.interactor import OrderCanceledInteractor
from services.model import OrderCanceledRequest


class OrderCanceledProcessor(UuidMessageProcessor):
    def __init__(
        self,
        order_canceled_interactor,    # type: OrderCanceledInteractor
    ):
        # type: (...) -> None

        super(OrderCanceledProcessor, self).__init__()
        self.order_canceled_interactor = order_canceled_interactor

    def parse_data(
        self,
        data,           # type: str
        event_type,     # type: str
    ):
        # type: (...) -> OrderCanceledRequest

        request_json = json.loads(data)

        return OrderCanceledRequest(
            pos_id=request_json["posId"],
            order_id=request_json["orderId"],
        )

    def call_business(
        self,
        request,  # type: OrderCanceledRequest
    ):
        # type: (...) -> None

        self.order_canceled_interactor.process_request(request)

    def format_response(
        self,
        message_bus,    # type: MessageBus
        message,        # type: Message
        event,          # type: Event
        request,        # type: OrderCanceledRequest
        result,         # type: None
    ):
        # type: (...) -> None

        return self._do_not_reply_to_asynchronous_event()

    def format_parse_exception(
        self,
        message_bus,    # type: MessageBus
        message,        # type: Message
        event,          # type: Event
        data,           # type: Any
        exception,      # type: Exception
    ):
        # type: (...) -> None

        return self._do_not_reply_to_asynchronous_event()

    def format_exception(
        self,
        message_bus,    # type: MessageBus
        message,        # type: Message
        event,          # type: Event
        input_obj,      # type: OrderCanceledRequest
        exception,      # type: Exception
    ):
        # type: (...) -> None

        return self._do_not_reply_to_asynchronous_event()

    @staticmethod
    def _do_not_reply_to_asynchronous_event():
        return None
